
/* File:        org.molgenis.omx/model/CategoricalValue.java
 * Copyright:   GBIC 2000-2013, all rights reserved
 * Date:        August 21, 2013
 * 
 * generator:   org.molgenis.generators.db.EntityImporterGen 4.0.0-testing
 *
 * 
 * THIS FILE HAS BEEN GENERATED, PLEASE DO NOT EDIT!
 */

package org.molgenis.omx.observ.value.db;

import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.concurrent.atomic.AtomicInteger;

import org.apache.commons.lang3.StringUtils;
import com.google.common.collect.Sets;
import org.apache.log4j.Logger;
import org.molgenis.framework.db.Database;
import org.molgenis.framework.db.DatabaseException;
import org.molgenis.framework.db.Database.DatabaseAction;
import org.molgenis.framework.db.EntityImporter;
import org.molgenis.io.processor.LowerCaseProcessor;
import org.molgenis.io.TupleReader;
import org.molgenis.util.tuple.Tuple;

import org.molgenis.omx.observ.Category;
import org.molgenis.omx.observ.value.CategoricalValue;


/**
 * Reads CategoricalValue from a delimited (csv) file, resolving xrefs to ids where needed, that is the tricky bit ;-)
 */
public class CategoricalValueEntityImporter implements EntityImporter
{
	private static final Logger logger = Logger.getLogger(CategoricalValueEntityImporter.class);
	
	private static int BATCH_SIZE = 10000;
	//foreign key map for xref 'value' (maps category.Identifier -> category.id)			
	final Map<String,Integer> valueKeymap = new TreeMap<String,Integer>();	
			
	/**
	 * Imports CategoricalValue from tab/comma delimited File
	 * @param db database to import into
	 * @param reader csv reader to load data from
	 * @param defaults to set default values for each row
	 * @param dbAction indicating wether to add,update,remove etc
	 * @param missingValues indicating what value in the csv is treated as 'null' (e.g. "" or "NA")
	 * @return number of elements imported
	 */
	@Override
	public int importEntity(TupleReader reader, Database db, DatabaseAction dbAction) throws IOException, DatabaseException 
	{
		// normalize column headers
		reader.addCellProcessor(new LowerCaseProcessor(true, false));
		//wrapper to count
		final AtomicInteger total = new AtomicInteger(0);
	try {
		//cache for entities of which xrefs couldn't be resolved (e.g. if there is a self-refence)
		//these entities can be updated with their xrefs in a second round when all entities are in the database
		List<CategoricalValue> categoricalValuesMissingRefs = new ArrayList<CategoricalValue>();
	
		//cache for objects to be imported from file (in batch)
		final List<CategoricalValue> categoricalValueList = new ArrayList<CategoricalValue>(BATCH_SIZE); // FIXME
		
		for(Tuple tuple : reader)
		{
			// skip empty rows
			if (!hasValues(tuple)) continue;
			
			//parse object, setting defaults and values from file
			CategoricalValue object = new CategoricalValue();
			object.set(tuple, false);				
			categoricalValueList.add(object);		
			
			//add to db when batch size is reached
			if(categoricalValueList.size() == BATCH_SIZE)
			{
				//resolve foreign keys and copy those entities that could not be resolved to the missingRefs list
				categoricalValuesMissingRefs.addAll(resolveForeignKeys(db, categoricalValueList));
				categoricalValueList.removeAll(categoricalValuesMissingRefs);
				
				//update objects in the database using xref_label defined secondary key(s) 'id' defined in xref_label
				db.update(categoricalValueList,dbAction, "id");
				
				//clear for next batch						
				categoricalValueList.clear();		
				
				//keep count
				total.set(total.get() + BATCH_SIZE);
				
				db.getEntityManager().flush();
				db.getEntityManager().clear();		
			}
		}
			
		//add remaining elements to the database
		if(!categoricalValueList.isEmpty())
		{
			total.set(total.get() + categoricalValueList.size());
			
			//resolve foreign keys, again keeping track of those entities that could not be solved
			categoricalValuesMissingRefs.addAll(resolveForeignKeys(db, categoricalValueList));
			categoricalValueList.removeAll(categoricalValuesMissingRefs);
			
			//update objects in the database using xref_label defined secondary key(s) 'id' defined in xref_label
			db.update(categoricalValueList,dbAction, "id");
		}
		
		//Try to resolve FK's for entities until all are resolved or we have more then 100 iterations
		List<CategoricalValue> categoricalValues = new ArrayList<CategoricalValue>(categoricalValuesMissingRefs);

		int iterationCount = 0;

		do
		{
			categoricalValuesMissingRefs = resolveForeignKeys(db, categoricalValuesMissingRefs);
			
			LinkedHashSet<CategoricalValue> differenceSet = new LinkedHashSet<CategoricalValue>();
			Sets.symmetricDifference(new LinkedHashSet<CategoricalValue>(categoricalValues), new LinkedHashSet<CategoricalValue>(categoricalValuesMissingRefs)).copyInto(differenceSet);
			List<CategoricalValue> resolvablecategoricalValues = new ArrayList<CategoricalValue>(differenceSet);
			
			categoricalValues.removeAll(resolvablecategoricalValues);
			
			db.update(resolvablecategoricalValues,dbAction, "id");

			if (iterationCount++ > 100)
			{
			String identifier = "";
			String name = "";
				for(CategoricalValue blaat : categoricalValuesMissingRefs){
					identifier = blaat.getValues().get("Identifier").toString();
					name = blaat.getValues().get("Name").toString();
				}
				throw new Exception(
						"Import of 'categoricalValue' entity failed:"
								+ "This is probably caused by a(n) 'categoricalValue' that has a reference but that does not exist."
								+"(identifier:"+identifier+", name:"+name+")");		
			}
		}
		while (categoricalValuesMissingRefs.size() > 0);

		logger.info("imported " + total.get() + " categoricalValue from CSV");

		} catch(Exception e) {throw new IOException(e);}
		return total.get();
	}	
	
	private boolean hasValues(Tuple tuple)
	{
		for (String colName : tuple.getColNames())
		{
			if (tuple.get(colName) != null) return true;
		}
		return false;
	}
	
	/**
	 * This method tries to resolve foreign keys (i.e. xref_field) based on the secondary key/key (i.e. xref_labels).
	 *
	 * @param db database
	 * @param categoricalValueList 
	 * @return the entities for which foreign keys cannot be resolved
	 */
	private List<CategoricalValue> resolveForeignKeys(Database db, List<CategoricalValue> categoricalValueList) throws Exception
	{
		//keep a list of CategoricalValue instances that miss a reference which might be resolvable later
		List<CategoricalValue> categoricalValuesMissingRefs = new ArrayList<CategoricalValue>();
	
		//resolve xref 'value' from category.Identifier -> category.id
		for(CategoricalValue o: categoricalValueList) 
		{
			if(o.getValue_Identifier() != null) 
				valueKeymap.put(o.getValue_Identifier(), null);
		}
		
		if(valueKeymap.size() > 0) 
		{
			List<Category> valueList = db.query(Category.class).in("Identifier",new ArrayList<Object>(valueKeymap.keySet())).find();
			for(Category xref :  valueList)
			{
				valueKeymap.put(xref.getIdentifier(), xref.getId());
			}
		}
		//update objects with foreign key values
		for(CategoricalValue o:  categoricalValueList)
		{
			while(true){
				//update xref Value
				if(o.getValue_Identifier() != null) 
				{
					String key = o.getValue_Identifier();
					if(valueKeymap.get(key) == null)
					{
						throw new Exception("Import of 'CategoricalValue' objects failed: cannot find Category for value_Identifier='"+o.getValue_Identifier()+"'");
					}
					o.setValue_Id(valueKeymap.get(key));
				}
				break;
			}
		}
		
		valueKeymap.clear();
		
		return categoricalValuesMissingRefs;
	}
}

