
/* File:        org.molgenis.omx/model/TextValue.java
 * Copyright:   GBIC 2000-2013, all rights reserved
 * Date:        August 21, 2013
 * 
 * generator:   org.molgenis.generators.db.EntityImporterGen 4.0.0-testing
 *
 * 
 * THIS FILE HAS BEEN GENERATED, PLEASE DO NOT EDIT!
 */

package org.molgenis.omx.observ.value.db;

import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

import org.apache.commons.lang3.StringUtils;
import com.google.common.collect.Sets;
import org.apache.log4j.Logger;
import org.molgenis.framework.db.Database;
import org.molgenis.framework.db.DatabaseException;
import org.molgenis.framework.db.Database.DatabaseAction;
import org.molgenis.framework.db.EntityImporter;
import org.molgenis.io.processor.LowerCaseProcessor;
import org.molgenis.io.TupleReader;
import org.molgenis.util.tuple.Tuple;

import org.molgenis.omx.observ.value.TextValue;


/**
 * Reads TextValue from a delimited (csv) file, resolving xrefs to ids where needed, that is the tricky bit ;-)
 */
public class TextValueEntityImporter implements EntityImporter
{
	private static final Logger logger = Logger.getLogger(TextValueEntityImporter.class);
	
	private static int BATCH_SIZE = 10000;
			
	/**
	 * Imports TextValue from tab/comma delimited File
	 * @param db database to import into
	 * @param reader csv reader to load data from
	 * @param defaults to set default values for each row
	 * @param dbAction indicating wether to add,update,remove etc
	 * @param missingValues indicating what value in the csv is treated as 'null' (e.g. "" or "NA")
	 * @return number of elements imported
	 */
	@Override
	public int importEntity(TupleReader reader, Database db, DatabaseAction dbAction) throws IOException, DatabaseException 
	{
		// normalize column headers
		reader.addCellProcessor(new LowerCaseProcessor(true, false));
		//wrapper to count
		final AtomicInteger total = new AtomicInteger(0);
	try {
		//cache for entities of which xrefs couldn't be resolved (e.g. if there is a self-refence)
		//these entities can be updated with their xrefs in a second round when all entities are in the database
		List<TextValue> textValuesMissingRefs = new ArrayList<TextValue>();
	
		//cache for objects to be imported from file (in batch)
		final List<TextValue> textValueList = new ArrayList<TextValue>(BATCH_SIZE); // FIXME
		
		for(Tuple tuple : reader)
		{
			// skip empty rows
			if (!hasValues(tuple)) continue;
			
			//parse object, setting defaults and values from file
			TextValue object = new TextValue();
			object.set(tuple, false);				
			textValueList.add(object);		
			
			//add to db when batch size is reached
			if(textValueList.size() == BATCH_SIZE)
			{
				//resolve foreign keys and copy those entities that could not be resolved to the missingRefs list
				textValuesMissingRefs.addAll(resolveForeignKeys(db, textValueList));
				textValueList.removeAll(textValuesMissingRefs);
				
				//update objects in the database using xref_label defined secondary key(s) 'id' defined in xref_label
				db.update(textValueList,dbAction, "id");
				
				//clear for next batch						
				textValueList.clear();		
				
				//keep count
				total.set(total.get() + BATCH_SIZE);
				
				db.getEntityManager().flush();
				db.getEntityManager().clear();		
			}
		}
			
		//add remaining elements to the database
		if(!textValueList.isEmpty())
		{
			total.set(total.get() + textValueList.size());
			
			//resolve foreign keys, again keeping track of those entities that could not be solved
			textValuesMissingRefs.addAll(resolveForeignKeys(db, textValueList));
			textValueList.removeAll(textValuesMissingRefs);
			
			//update objects in the database using xref_label defined secondary key(s) 'id' defined in xref_label
			db.update(textValueList,dbAction, "id");
		}
		
		//Try to resolve FK's for entities until all are resolved or we have more then 100 iterations
		List<TextValue> textValues = new ArrayList<TextValue>(textValuesMissingRefs);

		int iterationCount = 0;

		do
		{
			textValuesMissingRefs = resolveForeignKeys(db, textValuesMissingRefs);
			
			LinkedHashSet<TextValue> differenceSet = new LinkedHashSet<TextValue>();
			Sets.symmetricDifference(new LinkedHashSet<TextValue>(textValues), new LinkedHashSet<TextValue>(textValuesMissingRefs)).copyInto(differenceSet);
			List<TextValue> resolvabletextValues = new ArrayList<TextValue>(differenceSet);
			
			textValues.removeAll(resolvabletextValues);
			
			db.update(resolvabletextValues,dbAction, "id");

			if (iterationCount++ > 100)
			{
			String identifier = "";
			String name = "";
				for(TextValue blaat : textValuesMissingRefs){
					identifier = blaat.getValues().get("Identifier").toString();
					name = blaat.getValues().get("Name").toString();
				}
				throw new Exception(
						"Import of 'textValue' entity failed:"
								+ "This is probably caused by a(n) 'textValue' that has a reference but that does not exist."
								+"(identifier:"+identifier+", name:"+name+")");		
			}
		}
		while (textValuesMissingRefs.size() > 0);

		logger.info("imported " + total.get() + " textValue from CSV");

		} catch(Exception e) {throw new IOException(e);}
		return total.get();
	}	
	
	private boolean hasValues(Tuple tuple)
	{
		for (String colName : tuple.getColNames())
		{
			if (tuple.get(colName) != null) return true;
		}
		return false;
	}
	
	/**
	 * This method tries to resolve foreign keys (i.e. xref_field) based on the secondary key/key (i.e. xref_labels).
	 *
	 * @param db database
	 * @param textValueList 
	 * @return the entities for which foreign keys cannot be resolved
	 */
	private List<TextValue> resolveForeignKeys(Database db, List<TextValue> textValueList) throws Exception
	{
		//keep a list of TextValue instances that miss a reference which might be resolvable later
		List<TextValue> textValuesMissingRefs = new ArrayList<TextValue>();
	
		//update objects with foreign key values
		for(TextValue o:  textValueList)
		{
			while(true){
				break;
			}
		}
		
		
		return textValuesMissingRefs;
	}
}

